/**
	sound sketches:
	http://soundcloud.com/hems

	contact / help / anything, just drop a line:
	henrique@henriquematias.com
**/


/** outlets **/


// number of outlets
var outlets = 1;

// outlet that outputs bang when the sequencer reachs a filled toggle
var OUT_FILLED = 0;


/** constants **/


var DEBUG = false;

// space between toggle lines
var LINE_SPACE = 20;

// define initial Y position for first toggle line
var TOP_MARGIN = 200;

// color of the current toggle during sequence stepping
var SELECTED_COLOR = 8;


/** properties **/


// horizontal distance from one toggle to another
var distance;	

// total number of steps in a sequence
var steps;
var line_length;

var toggles;	// saves a list of created toggles
var selected_index; // current index of sequencer


/** initializing **/


selected_index = -1;

toggles       = [];
line_length   = 16;
distance      = 15;


/** methods **/


// save javascript values when saving patch
function save() {

    embedmessage( "set_steps", steps );
    embedmessage( "set_line_length", line_length );
    embedmessage( "set_distance", distance );

}

// user friendly message for non recognized messages
function anything() {

    post( "WARNING: you sent message named '" + messagename + "'" );
    post( "but its not implemented" );

}

function reset() {

    clear();

    toggles = [];
    selected_index = -1;

}

function dump_info() {
	
	post( "---dumping info---" );
	post( "\n");
	post( "total steps : " + toggles.length );
	post( "\n");
	post( "current step: " + ( selected_index + 1 ) );
	post( "\n");
	post( "---end of dump---\n" );
	
}

function set_steps( value ) {

    // steps cant be less than 1
    value = Math.max( 0, value );

    steps = value;

    redraw();

}

function set_distance( value ) {
    distance = value;
    
    redraw();
}

function set_line_length( value ) {

    // line_length can't be less than 1
    value = Math.max( 1, value );

    line_length = value;

    redraw();

}

/**
 * Clear than draw
 */
function redraw() {

    clear();

    var i;
    var x;
    var y;
    var toggle;

    for( i = 0; i < steps; i ++ ) {

        x = i % line_length * distance;
        y = Math.floor( i / line_length ) * LINE_SPACE;
        y = y + TOP_MARGIN;

        toggle = patcher.newobject( "toggle", x, y, 15, 0 ) ;

        toggles.push( toggle );

    }
}

/**
 * Simply removes all toggles, and reset toggle list ( Array )
 */
function clear() {

    var i;
    var toggle;

    for( i = 0; i < toggles.length; i++ ) {

        toggle = toggles[ i ];

        if( toggles[ i ] == null ) continue;

        if( toggle.maxclass != 'toggle' ) continue;

        patcher.remove( toggles[ i ] );

    }

	toggles = [];

}

/**
 * Moves the selection to next toggle
 */
function move_forward() {

    if( toggles.length == 0 ) return;

    var toggle;

	// gets last toggle and set background color to 0
    toggle = toggles[ selected_index ];

    if( selected_index > -1 )
            toggle.colorindex = 0;

	// select current toggle and set background color to SELECTED_COLOR
    selected_index ++;
    if( selected_index == toggles.length )
        selected_index = 0;

    toggle = toggles[ selected_index ];

    toggle.colorindex = SELECTED_COLOR;

	// check if toggle is ON, if so send a band throug outlet OUT_FILLED
    var is_selected;

    is_selected = ( toggle.getvalueof() == 1 );

    if( is_selected )
        outlet( OUT_FILLED, "bang" );
        
    if( DEBUG )
    	post( 'current step:' + selected_index + "\n");
    
}